@@ -15,7 +15,7 @@
 
 There should be a file 'ftesting.zcml' in the current directory.
 
-$Id: functional.py 83372 2008-02-01 13:26:00Z adamg $
+$Id: functional.py 90093 2008-08-21 20:35:37Z fdrake $
 """
 import logging
 import os.path
@@ -29,6 +29,7 @@ from Cookie import SimpleCookie
 from transaction import abort, commit
 from ZODB.DB import DB
 from ZODB.DemoStorage import DemoStorage
+from ZODB.interfaces import IDatabase
 
 from zope import component
 from zope.publisher.browser import BrowserRequest, setDefaultSkin
@@ -191,6 +192,7 @@ class FunctionalTestSetup(object):
                 BaseDatabaseFactory(name, self._base_storages)
                 for name in database_names
                 )[0][0]
+            self.dbstack = []
             self.app = Debugger(self.db, config_file)
 
             self.connection = None
@@ -231,30 +233,45 @@ class FunctionalTestSetup(object):
 
     base_storage = property(_get_base_storage, _set_base_storage)
 
+    def _close_databases(self):
+        # This is really careful to unregister the databases before attempting
+        # to close anything.  Zope Corporation has a couple of large
+        # multi-database applications that get bitten if we're not careful
+        # like this, but we've not been able to write a concise test case yet.
+        base = component.getGlobalSiteManager()
+        dbs = []
+        for name, db in list(component.getUtilitiesFor(IDatabase)):
+            ok = base.unregisterUtility(db, IDatabase, name)
+            assert ok
+            dbs.append(db)
+        if self.connection:
+            self.connection.close()
+            self.connection = None
+        for db in dbs:
+            db.close()
+
     def setUp(self):
         """Prepares for a functional test case."""
         # Tear down the old demo storages (if any) and create fresh ones
         abort()
-        for db in self.db.databases.itervalues():
-            db.close()
+        self.dbstack.append((self.db, self.connection))
+        self.connection = None
         self.db = self.app.db = multi_database(
             DerivedDatabaseFactory(name, self._base_storages)
             for name in self._database_names
             )[0][0]
-        self.connection = None
 
     def tearDown(self):
         """Cleans up after a functional test case."""
         abort()
-        if self.connection:
-            self.connection.close()
-            self.connection = None
-        for db in self.db.databases.itervalues():
-            db.close()
+        self._close_databases()
+        self.db, self.connection = self.dbstack.pop()
         setSite(None)
 
     def tearDownCompletely(self):
         """Cleans up the setup done by the constructor."""
+        self._close_databases()
+        assert self.dbstack == []
         zope.app.testing.setup.placefulTearDown()
         self._config_file = False
         self._database_names = None
